@@ -31,6 +31,8 @@
 
 int ai_family = AF_UNSPEC;
 
+extern int keep_running;
+
 static uint8_t invalid_cw[16] = { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 };
 
 int connect_client(int socktype, const char *hostname, const char *service) {
@@ -53,13 +55,13 @@ int connect_client(int socktype, const char *hostname, const char *service) {
 
 	int sockfd = -1;
 	struct addrinfo *ressave = res;
+	char str_addr[INET6_ADDRSTRLEN] = { 0 };
 	while (res) {
 		sockfd = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
 		if (sockfd > -1) {
+			my_inet_ntop(res->ai_family, res->ai_addr, str_addr, sizeof(str_addr));
 			if (do_connect(sockfd, res->ai_addr, res->ai_addrlen, 1000) < 0) {
-				char str_addr[INET6_ADDRSTRLEN];
-				my_inet_ntop(res->ai_family, res->ai_addr, str_addr, sizeof(str_addr));
-				ts_LOGf("CAM | Could not connect to server %s:%s (%s) | %s\n",
+				ts_LOGf("CAM | Error connecting to server %s port %s (addr=%s) | %s\n",
 					hostname, service, str_addr, strerror(errno));
 				close(sockfd);
 				sockfd = -1;
@@ -80,12 +82,15 @@ int connect_client(int socktype, const char *hostname, const char *service) {
 		setsockopt(sockfd, IPPROTO_TCP, TCP_NODELAY, &flag, sizeof(int));
 	}
 
-	ts_LOGf("CAM | Connected to fd:%d\n", sockfd);
+	ts_LOGf("CAM | Connected to server %s port %s (addr=%s fd=%d).\n",
+		hostname, service, str_addr, sockfd);
 
 	return sockfd;
 }
 
 static inline void camd_reconnect(struct camd *c) {
+	if (!keep_running)
+		return;
 	if (c->no_reconnect)
 		return;
 	c->ops.reconnect(c);
@@ -118,6 +123,9 @@ static int camd_recv_cw(struct ts *ts) {
 	ret = c->ops.get_cw(c, &ca_id, &idx, c->key->cw);
 	gettimeofday(&tv2, NULL);
 
+	if (!keep_running)
+		return 0;
+
 	if (ret <= 0) {
 		if (ret == -1) { // Fatal error it is better to reconnect to server.
 			ts_LOGf("ERR | No code word has been received (ret = %d)\n", ret);
@@ -212,6 +220,8 @@ static int camd_send_emm(struct ts *ts, struct camd_msg *msg) {
 }
 
 static void camd_do_msg(struct camd_msg *msg) {
+	if (!keep_running)
+		goto OUT;
 	if (msg->type == EMM_MSG) {
 		msg->ts->emm_seen_count++;
 		if (camd_send_emm(msg->ts, msg) > 0)
@@ -222,7 +232,7 @@ static void camd_do_msg(struct camd_msg *msg) {
 		if (camd_send_ecm(msg->ts, msg) > 0)
 			msg->ts->ecm_processed_count++;
 	}
-
+OUT:
 	camd_msg_free(&msg);
 }
 
@@ -248,7 +258,7 @@ static void *camd_thread(void *in_ts) {
 
 	set_thread_name("tsdec-camd");
 
-	while (1) {
+	while (keep_running) {
 		struct camd_msg *msg;
 		void *req = queue_get(ts->camd.req_queue); // Waits...
 		if (ts->camd_stop)
