@@ -72,7 +72,11 @@ void data_init(struct ts *ts) {
 
 	ts->debug_level = 0;
 	ts->req_CA_sys  = CA_CONAX;
-	ts->emm_send    = 0;
+
+	ts->process_ecm = 1;
+	ts->process_emm = 0;
+	ts->output_stream = 1;
+
 	ts->pid_filter  = 1;
 
 	ts->emm_report_interval = 60;
@@ -81,6 +85,9 @@ void data_init(struct ts *ts) {
 	ts->ecm_report_interval = 60;
 	ts->ecm_last_report     = time(NULL);
 
+	ts->irdeto_ecm_idx         = 0;
+	ts->irdeto_ecm_filter_type = IRDETO_FILTER_IDX;
+
 	ts->cw_warn_sec = 60;
 	ts->cw_last_warn= time(NULL);
 	ts->cw_last_warn= ts->cw_last_warn + ts->cw_warn_sec;
