@@ -17,6 +17,7 @@
 
 #include <pthread.h>
 #include <limits.h>
+#include <stdbool.h>
 
 #include <openssl/aes.h>
 #include <openssl/des.h>
@@ -29,7 +30,7 @@
 #define FRAME_SIZE 1316
 
 // How much seconds to assume the key is valid
-#define KEY_VALID_TIME 10
+#define KEY_VALID_TIME 30
 
 #define EMM_QUEUE_HARD_LIMIT 10000
 #define EMM_QUEUE_SOFT_LIMIT 1000
@@ -179,6 +180,8 @@ struct io {
 	int					tos;
 	struct in_addr		intf;
 	int					v6_if_index;
+	// Used for input
+	struct in_addr		isrc;
 };
 
 struct packet_buf {
@@ -198,14 +201,27 @@ enum filter_action {
 	FILTER_REJECT     = 4,
 };
 
+enum filter_type {
+	FILTER_TYPE_DATA = 0,				// Compare data at offset X
+	FILTER_TYPE_MASK = 1,				// Compare data + mask
+	FILTER_TYPE_LENGTH = 2,				// Compare section length /EMM[2]/)
+};
+
 struct filter {
-	enum filter_action action;			// If set to 1, the filter is negative else the filter is positive
+	enum filter_action action;
+	enum filter_type type;
 	uint8_t		offset;					// Offset into EMM
-	uint8_t		data_len;				// Filter length
-	uint8_t		data[MAX_FILTER_LEN];	// Filter bytes
+	uint8_t		filter_len;				// Filter length
+	uint8_t		data[MAX_FILTER_LEN];	// Data | Matched bytes
+	uint8_t		mask[MAX_FILTER_LEN];	// Mask bytes
 	char		name[MAX_FILTER_NAME];	// Filter name (default: NO_NAME)
 };
 
+struct chid {
+	int			seen;
+	uint16_t	chid;
+};
+
 #define MAX_PIDS 8192
 
 struct ts {
@@ -270,14 +286,26 @@ struct ts {
 
 	enum CA_system		req_CA_sys;
 
-	int					emm_send;
-	int					emm_only;
+	bool				output_stream;		// Decode and output the decoded stream
+	bool				process_ecm;		// Process ECM packets (and send them to CAMD)
+	bool				process_emm;		// Process EMM packets (and send them to CAMD)
+
 	int					pid_filter;
 	int					eit_passthrough;
 	int					tdt_passthrough;
 	int					nit_passthrough;
 
-	uint8_t				irdeto_ecm;
+	uint8_t				irdeto_ecm_idx;
+	uint16_t			irdeto_ecm_chid;
+
+	enum {
+		IRDETO_FILTER_IDX,
+		IRDETO_FILTER_CHID,
+	} irdeto_ecm_filter_type;
+
+	struct chid			irdeto_chid[0xff];
+	uint8_t				irdeto_max_chids;
+
 	int					ecm_cw_log;
 
 	int					rtp_input;
