@@ -194,12 +194,26 @@ enum CA_system ts_get_CA_sys(uint16_t CA_id) {
 	if (CA_id >= 0x0B00 && CA_id <= 0x0BFF) return CA_CONAX;
 	if (CA_id >= 0x0D00 && CA_id <= 0x0DFF) return CA_CRYPTOWORKS;
 	if (CA_id >= 0x1800 && CA_id <= 0x18FF) return CA_NAGRA;
-	if (CA_id >= 0x4AE0 && CA_id <= 0x4AE1) return CA_DRECRYPT;
-	if (CA_id == 0x5581 || CA_id == 0x4AEE) return CA_BULCRYPT;
+	switch (CA_id) {
+		case 0x4ABF: return CA_DGCRYPT;
+
+		case 0x4AE0: return CA_DRECRYPT;
+		case 0x4AE1: return CA_DRECRYPT;
+
+		case 0x5581: return CA_BULCRYPT;
+		case 0x4AEE: return CA_BULCRYPT;
+
+		case 0x5501: return CA_GRIFFIN;
+		case 0x5504: return CA_GRIFFIN;
+		case 0x5506: return CA_GRIFFIN;
+		case 0x5508: return CA_GRIFFIN;
+		case 0x5509: return CA_GRIFFIN;
+		case 0x550E: return CA_GRIFFIN;
+		case 0x5511: return CA_GRIFFIN;
+	}
 	return CA_UNKNOWN;
 }
 
-
 char * ts_get_CA_sys_txt(enum CA_system CA_sys) {
 	switch (CA_sys) {
 		case CA_SECA:			return "SECA";
@@ -211,6 +225,8 @@ char * ts_get_CA_sys_txt(enum CA_system CA_sys) {
 		case CA_NAGRA:			return "NAGRA";
 		case CA_DRECRYPT:		return "DRE-CRYPT";
 		case CA_BULCRYPT:		return "BULCRYPT";
+		case CA_GRIFFIN:		return "GRIFFIN";
+		case CA_DGCRYPT:		return "DGCRYPT";
 		case CA_UNKNOWN:		return "UNKNOWN";
 	}
 	return "UNKNOWN";
