@@ -12,6 +12,8 @@
  * GNU General Public License (COPYING file) for more details.
  *
  */
+#include <string.h>
+
 #include "data.h"
 #include "tables.h"
 #include "camd.h"
@@ -173,6 +175,11 @@ void process_pmt(struct ts *ts, uint16_t pid, uint8_t *ts_packet) {
 	} else {
 		ts_LOGf("*** | ERROR: Can't detect ECM pid.\n");
 	}
+
+	if (ts->req_CA_sys == CA_IRDETO) {
+		memset(ts->irdeto_chid, 0, sizeof(ts->irdeto_chid));
+		ts->irdeto_max_chids = 0;
+	}
 }
 
 static int sdt_parse_service_name_desc(
@@ -274,9 +281,6 @@ static void __process_emm(struct ts *ts, uint16_t pid, uint8_t *ts_packet) {
 
 	ts->emm_input_count++;
 
-	if (!ts->emm_send)
-		return;
-
 	ts->emm = ts_privsec_push_packet(ts->emm, ts_packet);
 	if (!ts->emm->initialized)
 		return;
@@ -317,8 +321,30 @@ static void __process_ecm(struct ts *ts, uint16_t pid, uint8_t *ts_packet) {
 		return;
 
 	if (ts->req_CA_sys == CA_IRDETO) {
-		int type = ts->ecm->section_header->section_data[4];
-		if (type != ts->irdeto_ecm) {
+		uint8_t idx     = ts->ecm->section_header->section_data[4];
+		uint8_t max_idx = ts->ecm->section_header->section_data[5];
+		uint16_t chid   = (ts->ecm->section_header->section_data[6] << 8) | ts->ecm->section_header->section_data[7];
+
+		if (max_idx != ts->irdeto_max_chids) {
+			memset(ts->irdeto_chid, 0, sizeof(ts->irdeto_chid));
+			ts->irdeto_max_chids = max_idx;
+		}
+
+		bool ecm_ok = false;
+		const char *filter_type;
+		switch (ts->irdeto_ecm_filter_type) {
+		case IRDETO_FILTER_IDX : ecm_ok = (idx == ts->irdeto_ecm_idx); filter_type = " BY IDX"; break;
+		case IRDETO_FILTER_CHID: ecm_ok = (chid == ts->irdeto_ecm_chid); filter_type = " BY CHID"; break;
+		}
+
+		if (ts->irdeto_chid[idx].seen < 1) {
+			ts_LOGf("CAS | Seen Irdeto CHID 0x%04x (idx %u/%u)%s%s\n", chid, idx, max_idx,
+				ecm_ok ? " *SELECTED*" : "",
+				ecm_ok ? filter_type   : "");
+			ts->irdeto_chid[idx].seen++;
+		}
+
+		if (!ecm_ok) {
 			ts_privsec_clear(ts->ecm);
 			return;
 		}
@@ -336,7 +362,7 @@ static void __process_ecm(struct ts *ts, uint16_t pid, uint8_t *ts_packet) {
 	int duplicate = ts_privsec_is_same(ts->ecm, ts->last_ecm);
 	if (duplicate && !ts->is_cw_error)
 		ts->ecm_duplicate_count++;
-	if (!ts->ecm_change_time.tv_sec)
+	if (!ts->ecm_change_time.tv_sec && !ts->ecm_change_time.tv_usec) // The first time
 		gettimeofday(&ts->ecm_change_time, NULL);
 	if (!duplicate || ts->is_cw_error) {
 		if (ts->ecm_cw_log) {
@@ -353,8 +379,8 @@ static void __process_ecm(struct ts *ts, uint16_t pid, uint8_t *ts_packet) {
 				sec->table_id,
 				sec->section_data_len,
 				dump);
-			gettimeofday(&ts->ecm_change_time, NULL);
 		}
+		gettimeofday(&ts->ecm_change_time, NULL);
 		ts->is_cw_error = 0;
 		camd_process_packet(ts, camd_msg_alloc(ECM_MSG, ts->ecm_caid, ts->service_id, sec->section_data, sec->section_data_len));
 	} else if (ts->debug_level >= 3) {
@@ -381,7 +407,7 @@ static void __process_ecm(struct ts *ts, uint16_t pid, uint8_t *ts_packet) {
 void process_ecm(struct ts *ts, uint16_t pid, uint8_t *ts_packet) {
 	int section_end;
 
-	if (ts->emm_only)
+	if (!ts->process_ecm)
 		return;
 
 	if (!ts->ecm_pid || ts->ecm_pid != pid)
@@ -410,7 +436,7 @@ process_psi:
 void process_emm(struct ts *ts, uint16_t pid, uint8_t *ts_packet) {
 	int section_end;
 
-	if (!ts->emm_pid || ts->emm_pid != pid)
+	if (!ts->process_emm)
 		return;
 
 process_psi:
