@@ -42,7 +42,7 @@
 #define PROGRAM_NAME "tsdecrypt"
 static const char *program_id = PROGRAM_NAME " v" VERSION " (" GIT_VER ", build " BUILD_ID " " DLIB ")";
 
-static int keep_running = 1;
+int keep_running = 1;
 static FILE *log_file = NULL;
 static char *log_filename = NULL;
 static int local_syslog = 0;
@@ -79,9 +79,9 @@ static void LOG_func(const char *msg) {
 		LOG(msg);
 }
 
-static const char short_options[] = "i:d:N:Sl:L:F:I:RzM:T:W:O:o:t:rk:g:upwxyc:C:Y:Q:A:s:U:P:B:46eZ:Ef:a:X:H:G:KJ:D:jbhVn:m:";
+static const char short_options[] = "i:d:N:Sl:L:F:I:1:RzM:T:W:O:o:t:rk:g:upwxyc:C:Y:Q:A:s:U:P:B:46eZ:Ef:a:X:vqH:G:2:KJ:D:jbhVn:m:";
 
-// Unused short options: aqv01235789
+// Unused short options: 035789
 static const struct option long_options[] = {
 	{ "ident",				required_argument, NULL, 'i' },
 	{ "daemon",				required_argument, NULL, 'd' },
@@ -92,6 +92,7 @@ static const struct option long_options[] = {
 	{ "notify-program",		required_argument, NULL, 'N' },
 
 	{ "input",				required_argument, NULL, 'I' },
+	{ "input-source",		required_argument, NULL, '1' },
 	{ "input-rtp",			no_argument,       NULL, 'R' },
 	{ "input-ignore-disc",	no_argument,       NULL, 'z' },
 	{ "input-service",		required_argument, NULL, 'M' },
@@ -130,11 +131,15 @@ static const struct option long_options[] = {
 	{ "emm-filter",			required_argument, NULL, 'a' },
 
 	{ "ecm-pid",			required_argument, NULL, 'X' },
+	{ "ecm-only",			no_argument,       NULL, 'v' },
 	{ "ecm-report-time",	required_argument, NULL, 'H' },
 	{ "ecm-irdeto-type",	required_argument, NULL, 'G' },
+	{ "ecm-irdeto-chid",	required_argument, NULL, '2' },
 	{ "ecm-no-log",			no_argument      , NULL, 'K' },
 	{ "cw-warn-time",		required_argument, NULL, 'J' },
 
+	{ "ecm-and-emm-only",	no_argument,       NULL, 'q' },
+
 	{ "debug",				required_argument, NULL, 'D' },
 	{ "pid-report",			no_argument,       NULL, 'j' },
 	{ "bench",				no_argument,       NULL, 'b' },
@@ -164,6 +169,7 @@ static void show_help(struct ts *ts) {
 	printf("                            .    -I [ff01::1111]:5000 (v6 multicast)\n");
 	printf("                            .    -I file://in.ts      (read from file)\n");
 	printf("                            . By default the input is stdin.\n");
+	printf(" -1 --input-source <ipaddr> | Set multicast input source ip.\n");
 	printf(" -R --input-rtp             | Enable RTP input\n");
 	printf(" -z --input-ignore-disc     | Do not report discontinuty errors in input.\n");
 	printf(" -M --input-service <srvid> | Choose service id when input is MPTS.\n");
@@ -193,7 +199,8 @@ static void show_help(struct ts *ts) {
 	printf(" -c --ca-system <ca_sys>    | Process input EMM/ECM from <ca_sys>.\n");
 	printf("                            | Valid systems are: CONAX (default), CRYPTOWORKS,\n");
 	printf("                            .   IRDETO, SECA (MEDIAGUARD), VIACCESS,\n");
-	printf("                            .   VIDEOGUARD (NDS), NAGRA, DRECRYPT and BULCRYPT.\n");
+	printf("                            .   VIDEOGUARD (NDS), NAGRA, DRECRYPT, BULCRYPT,\n");
+	printf("                            .   GRIFFIN and DGCRYPT.\n");
 	printf(" -C --caid <caid>           | Set CAID. Default: Taken from --ca-system.\n");
 	printf(" -Y --const-cw <codeword>   | Set constant code word for decryption.\n");
 	printf("                            . Example cw: a1a2a3a4a5a6a7a8b1b2b3b4b5b6b7b8\n");
@@ -218,7 +225,7 @@ static void show_help(struct ts *ts) {
 	printf(" -6 --ipv6                  | Use only IPv6 addresses of the camd server.\n");
 	printf("\n");
 	printf("EMM options:\n");
-	printf(" -e --emm                   | Enable sending EMM's to CAMD. Default: %s\n", ts->emm_send ? "enabled" : "disabled");
+	printf(" -e --emm                   | Enable sending EMM's to CAMD. Default: %s\n", ts->process_emm ? "enabled" : "disabled");
 	printf(" -E --emm-only              | Send only EMMs to CAMD, skipping ECMs and without\n");
 	printf("                            .   decoding the input stream.\n");
 	printf(" -Z --emm-pid <pid>         | Force EMM pid. Default: none\n");
@@ -231,14 +238,21 @@ static void show_help(struct ts *ts) {
 	printf("\n");
 	printf("ECM options:\n");
 	printf(" -X --ecm-pid <pid>         | Force ECM pid. Default: none\n");
+	printf(" -v --ecm-only              | Send only ECMs to CAMD, skipping EMMs and without\n");
+	printf("                            .   decoding the input stream.\n");
 	printf(" -H --ecm-report-time <sec> | Report each <sec> how much ECMs and CWs have been\n");
 	printf("                            .   processed/skipped. Set <sec> to 0 to disable\n");
 	printf("                            .   the reports. Default: %d sec\n", ts->ecm_report_interval);
-	printf(" -G --ecm-irdeto-type <int> | Process IRDETO ECMs with type X /0-3/. Default: %d\n", ts->irdeto_ecm);
+	printf(" -G --ecm-irdeto-type <int> | Process IRDETO ECMs with index X /0-255/\n");
+	printf("                            .   It is better to use --ecm-irdeto-chid option!\n");
+	printf(" -2 --ecm-irdeto-chid <int> | Set CHID to filter Irdeto ECMs (Default: 0x0000).\n");
 	printf(" -K --ecm-no-log            | Disable ECM and code words logging.\n");
 	printf(" -J --cw-warn-time <sec>    | Warn if no valid code word has been received.\n");
 	printf("                            .   Set <sec> to 0 to disable. Default: %d sec\n", ts->cw_warn_sec);
 	printf("\n");
+	printf(" -q --ecm-and-emm-only      | Send ECMs and EMMs to CAMD but do not decode\n");
+	printf("                            .   the input stream.\n");
+	printf("\n");
 	printf("Logging options:\n");
 	printf(" -S --syslog                | Log messages using syslog.\n");
 	printf(" -l --syslog-host <host>    | Syslog server address. Default: disabled\n");
@@ -328,6 +342,12 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 			case 'I': // --input
 				input_addr_err = !parse_io_param(&ts->input, optarg, O_RDONLY, 0);
 				break;
+			case '1': // --input-source
+				if (!inet_aton(optarg, &ts->input.isrc)) {
+					fprintf(stderr, "ERROR: Can't parse input-source IP address: %s\n", optarg);
+					exit(EXIT_FAILURE);
+				}
+				break;
 			case 'R': // --input-rtp
 				ts->rtp_input = !ts->rtp_input;
 				break;
@@ -401,6 +421,10 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 					ts->req_CA_sys = CA_DRECRYPT;
 				else if (strcasecmp("BULCRYPT", optarg) == 0)
 					ts->req_CA_sys = CA_BULCRYPT;
+				else if (strcasecmp("GRIFFIN", optarg) == 0)
+					ts->req_CA_sys = CA_GRIFFIN;
+				else if (strcasecmp("DGCRYPT", optarg) == 0)
+					ts->req_CA_sys = CA_DGCRYPT;
 				else
 					ca_err = 1;
 				break;
@@ -493,14 +517,15 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 				break;
 
 			case 'e': // --emm
-				ts->emm_send = !ts->emm_send;
+				ts->process_emm = !ts->process_emm;
 				break;
 			case 'Z': // --emm-pid
 				ts->forced_emm_pid = strtoul(optarg, NULL, 0) & 0x1fff;
 				break;
 			case 'E': // --emm-only
-				ts->emm_only = 1;
-				ts->emm_send = 1;
+				ts->process_emm = 1;
+				ts->process_ecm = 0;
+				ts->output_stream = 0;
 				break;
 			case 'f': // --emm-report-time
 				ts->emm_report_interval = strtoul(optarg, NULL, 10);
@@ -524,13 +549,23 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 			case 'X': // --ecm-pid
 				ts->forced_ecm_pid = strtoul(optarg, NULL, 0) & 0x1fff;
 				break;
+			case 'v': // --ecm-only
+				ts->process_emm = 0;
+				ts->process_ecm = 1;
+				ts->output_stream = 0;
+				break;
 			case 'H': // --ecm-report-time
 				ts->ecm_report_interval = strtoul(optarg, NULL, 10);
 				if (ts->ecm_report_interval > 86400)
 					ts->ecm_report_interval = 86400;
 				break;
 			case 'G': // --ecm-irdeto-type
-				ts->irdeto_ecm = atoi(optarg);
+				ts->irdeto_ecm_idx = strtoul(optarg, NULL, 0);
+				ts->irdeto_ecm_filter_type = IRDETO_FILTER_IDX;
+				break;
+			case '2': // --ecm-irdeto-chid
+				ts->irdeto_ecm_chid = strtoul(optarg, NULL, 0);
+				ts->irdeto_ecm_filter_type = IRDETO_FILTER_CHID;
 				break;
 			case 'K': // --ecm-no-log
 				ts->ecm_cw_log = !ts->ecm_cw_log;
@@ -541,6 +576,11 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 					ts->cw_warn_sec = 86400;
 				ts->cw_last_warn= ts->cw_last_warn + ts->cw_warn_sec;
 				break;
+			case 'q': // --ecm-and-emm-only
+				ts->process_emm = 1;
+				ts->process_ecm = 1;
+				ts->output_stream = 0;
+				break;
 
 			case 'D': // --debug
 				ts->debug_level = atoi(optarg);
@@ -600,10 +640,9 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 		ts->output.type = FILE_IO;
 		ts->output.fd = 1;
 		ts->pid_filter = 0;
-		ts->emm_send = 1;
-		ts->emm_report_interval = 0;
-		ts->ecm_report_interval = 0;
-		ts->cw_warn_sec = 0;
+		ts->process_ecm = 0;
+		ts->process_emm = 0;
+		ts->output_stream = 0;
 		ts->camd.no_reconnect = 1;
 		ts->camd.check_emm_errors = 1;
 		ts->emm_filters_num = 0;
@@ -612,13 +651,9 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 	// Constant codeword is special. Disable conflicting options
 	if (ts->camd.constant_codeword) {
 		server_err = 0; // No server settings are required
-		ts->forced_caid = 0;
-		ts->forced_ecm_pid = 0;
-		ts->forced_emm_pid = 0;
-		ts->emm_send = 0;
-		ts->emm_report_interval = 0;
-		ts->ecm_report_interval = 0;
-		ts->cw_warn_sec = 0;
+		ts->process_ecm = 0;
+		ts->process_emm = 0;
+		ts->output_stream = 1;
 	}
 
 	if (ident_err || ca_err || server_err || input_addr_err || output_addr_err || ts->input.type == WTF_IO || ts->output.type == WTF_IO) {
@@ -689,6 +724,7 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 		ts_LOGf("Input addr : %s://%s:%s/\n",
 			ts->rtp_input ? "rtp" : "udp",
 			ts->input.hostname, ts->input.service);
+		ts_LOGf("Input src  : %s\n", inet_ntoa(ts->input.isrc));
 		if (ts->input_buffer_time) {
 			ts_LOGf("Input buff : %u ms\n", ts->input_buffer_time);
 		}
@@ -706,11 +742,14 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 	if (ts->forced_service_id)
 		ts_LOGf("Service id : 0x%04x (%d)\n",
 			ts->forced_service_id, ts->forced_service_id);
-	if (ts->req_CA_sys == CA_IRDETO)
-		ts_LOGf("Irdeto ECM : %d\n", ts->irdeto_ecm);
+	if (ts->req_CA_sys == CA_IRDETO) {
+		switch (ts->irdeto_ecm_filter_type) {
+		case IRDETO_FILTER_IDX : ts_LOGf("Irdeto ECM : Index: 0x%02x (%d)\n", ts->irdeto_ecm_idx, ts->irdeto_ecm_idx); break;
+		case IRDETO_FILTER_CHID: ts_LOGf("Irdeto ECM : CHID: 0x%04x (%d)\n", ts->irdeto_ecm_chid, ts->irdeto_ecm_chid); break;
+		}
+	}
 
-	if (!ts->emm_only)
-	{
+	if (ts->output_stream) {
 		if (ts->output.type == NET_IO) {
 			ts_LOGf("Output addr: %s://%s:%s/\n",
 				ts->rtp_output ? "rtp" : "udp",
@@ -727,16 +766,13 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 				RAND_bytes((unsigned char *)&(ts->rtp_seqnum), 2);
 			}
 		} else if (ts->output.type == FILE_IO) {
-			if (!packet_from_file)
-				ts_LOGf("Output file: %s\n", ts->output.fd == 1 ? "STDOUT" : ts->output.fname);
-		}
-		if (!packet_from_file) {
-			ts_LOGf("Out filter : %s (%s)%s\n",
-				ts->pid_filter ? "enabled" : "disabled",
-				ts->pid_filter ? "output only service related PIDs" : "output everything",
-				ts->no_output_on_error ? " (No output on CW error)" : ""
-			);
+			ts_LOGf("Output file: %s\n", ts->output.fd == 1 ? "STDOUT" : ts->output.fname);
 		}
+		ts_LOGf("Out filter : %s (%s)%s\n",
+			ts->pid_filter ? "enabled" : "disabled",
+			ts->pid_filter ? "output only service related PIDs" : "output everything",
+			ts->no_output_on_error ? " (No output on CW error)" : ""
+		);
 		if (ts->pid_filter) {
 			if (ts->nit_passthrough)
 				ts_LOGf("Out filter : Pass through NIT.\n");
@@ -745,7 +781,13 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 			if (ts->tdt_passthrough)
 				ts_LOGf("Out filter : Pass through TDT/TOT.\n");
 		}
+		ts_LOGf("TS discont : %s\n", ts->ts_discont ? "report" : "ignore");
+		ts->threaded = !(ts->input.type == FILE_IO && ts->input.fd != 0);
+		ts_LOGf("Decoding   : %s\n", ts->threaded ? "threaded" : "single thread");
+	} else {
+		ts_LOGf("Decoding   : disabled\n");
 	}
+
 	if (!ts->camd.constant_codeword) {
 		ts_LOGf("CAMD proto : %s\n", ts->camd.ops.ident);
 		ts_LOGf("CAMD addr  : %s:%s%s\n", ts->camd.hostname, ts->camd.service,
@@ -759,34 +801,18 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 			ts_LOGf("CAMD deskey: %s\n", ts->camd.newcamd.hex_des_key);
 	}
 
-	if (!packet_from_file) {
-		ts_LOGf("TS discont : %s\n", ts->ts_discont ? "report" : "ignore");
-		ts->threaded = !(ts->input.type == FILE_IO && ts->input.fd != 0);
-	}
+	if (!packet_from_file)
+		ts_LOGf("EMM process: %s\n", ts->process_emm ? "Yes" : "No");
 
-	if (!packet_from_file && !ts->emm_only) {
-		ts_LOGf("Decoding   : %s\n", ts->threaded ? "threaded" : "single thread");
-	}
-
-	if (!packet_from_file) {
-		if (ts->emm_send && ts->emm_report_interval)
-			ts_LOGf("EMM report : %d sec\n", ts->emm_report_interval);
-		if (ts->emm_send && ts->emm_report_interval == 0)
-			ts_LOGf("EMM report : disabled\n");
+	if (ts->process_emm) {
 		if (ts->forced_emm_pid)
 			ts_LOGf("EMM pid    : 0x%04x (%d)\n", ts->forced_emm_pid, ts->forced_emm_pid);
-	}
-	if (ts->emm_only) {
-		ts_LOGf("EMM only   : %s\n", ts->emm_only ? "yes" : "no");
-	} else {
-		if (!packet_from_file) {
-			if (!ts->camd.constant_codeword)
-				ts_LOGf("EMM send   : %s\n", ts->emm_send   ? "enabled" : "disabled");
-			ts_LOGf("Decoding   : %s\n", ts->threaded ? "threaded" : "single thread");
-		}
-	}
 
-	if (ts->emm_send) {
+		if (ts->emm_report_interval)
+			ts_LOGf("EMM report : %d sec\n", ts->emm_report_interval);
+		else
+			ts_LOGf("EMM report : disabled\n");
+
 		for (i = 0; i < ts->emm_filters_num; i++) {
 			char tmp[512];
 			filter_dump(&ts->emm_filters[i], tmp, sizeof(tmp));
@@ -794,20 +820,24 @@ static void parse_options(struct ts *ts, int argc, char **argv) {
 		}
 	}
 
-	if (!packet_from_file) {
-		if (!ts->emm_only && ts->ecm_report_interval)
-			ts_LOGf("ECM report : %d sec\n", ts->emm_report_interval);
-		if (!ts->emm_only && ts->ecm_report_interval == 0 && !ts->camd.constant_codeword)
-			ts_LOGf("ECM report : disabled\n");
+	if (!packet_from_file)
+		ts_LOGf("ECM process: %s\n", ts->process_ecm ? "Yes" : "No");
+
+	if (ts->process_ecm) {
 		if (ts->forced_ecm_pid)
 			ts_LOGf("ECM pid    : 0x%04x (%d)\n", ts->forced_ecm_pid, ts->forced_ecm_pid);
 
-		if (!ts->emm_only && ts->cw_warn_sec)
+		if (ts->ecm_report_interval)
+			ts_LOGf("ECM report : %d sec\n", ts->emm_report_interval);
+		else
+			ts_LOGf("ECM report : disabled\n");
+
+		if (ts->cw_warn_sec)
 			ts_LOGf("CW warning : %d sec\n", ts->cw_warn_sec);
-		if (!ts->emm_only && ts->cw_warn_sec && !ts->camd.constant_codeword)
+		else
 			ts_LOGf("CW warning : disabled\n");
 
-		if (!ts->ecm_cw_log && !ts->camd.constant_codeword)
+		if (!ts->ecm_cw_log)
 			ts_LOGf("ECM/CW log : disabled\n");
 	}
 
@@ -865,7 +895,7 @@ static void do_reports(struct ts *ts) {
 	static int first_emm_report = 1;
 	static int first_ecm_report = 1;
 	time_t now = time(NULL);
-	if (ts->emm_send && ts->emm_report_interval) {
+	if (ts->process_emm && ts->emm_report_interval) {
 		if (first_emm_report && now >= ts->emm_last_report) {
 			first_emm_report = 0;
 			ts->emm_last_report -= FIRST_REPORT_SEC;
@@ -874,7 +904,7 @@ static void do_reports(struct ts *ts) {
 			report_emms(ts, now);
 		}
 	}
-	if (!ts->emm_only && ts->ecm_report_interval) {
+	if (ts->process_ecm && ts->ecm_report_interval) {
 		if (first_ecm_report && now >= ts->ecm_last_report) {
 			first_ecm_report = 0;
 			ts->ecm_last_report -= FIRST_REPORT_SEC;
@@ -884,26 +914,36 @@ static void do_reports(struct ts *ts) {
 		}
 	}
 
-	if (!ts->emm_only && !ts->key.is_valid_cw) {
+	if (ts->process_ecm && !ts->key.is_valid_cw) {
 		if (ts->cw_warn_sec && now >= ts->cw_next_warn) {
 			report_cw_warn(ts, now);
 		}
 	}
 }
 
+static struct ts ts;
+
 void signal_quit(int sig) {
 	if (!keep_running)
 		raise(sig);
 	keep_running = 0;
 	ts_LOGf("Killed %s with signal %d\n", program_id, sig);
+	if (ts.input.type == NET_IO)
+		shutdown_fd(&ts.input.fd);
+	if (ts.output.type == NET_IO)
+		shutdown_fd(&ts.output.fd);
 	signal(sig, SIG_DFL);
 }
 
+void signal_alarm(int sig) {
+	(void)sig;
+	raise(SIGINT);
+}
+
 #define RTP_HDR_SZ  12
 
 static uint8_t ts_packet[FRAME_SIZE + RTP_HDR_SZ];
 static uint8_t rtp_hdr[2][RTP_HDR_SZ];
-static struct ts ts;
 
 int main(int argc, char **argv) {
 	ssize_t readen;
@@ -1001,6 +1041,8 @@ int main(int argc, char **argv) {
 				}
 			}
 			set_log_io_errors(1);
+			if (!keep_running)
+				break;
 			if (readen < 0) {
 				ts_LOGf("--- | Input read timeout.\n");
 				if (!ntimeouts) {
@@ -1028,12 +1070,15 @@ int main(int argc, char **argv) {
 				fwrite(ts_packet, readen, 1, ts.input_dump_file);
 			process_packets(&ts, ts_packet, readen);
 		}
-		if (!keep_running)
-			break;
-	} while (have_data);
+	} while (have_data && keep_running);
 EXIT:
 	camd_stop(&ts);
 
+	// If pthread_join failes make sure we exit...
+	signal(SIGINT , SIG_DFL);
+	signal(SIGTERM, SIG_DFL);
+	signal(SIGALRM, signal_alarm);
+	alarm(2);
 	if (ts.threaded) {
 		ts.decode_stop = 1;
 		ts.write_stop = 1;
