@@ -50,7 +50,7 @@ static int is_multicast(struct sockaddr_storage *addr) {
 
 extern int ai_family;
 
-static int bind_addr(const char *hostname, const char *service, int socktype, struct sockaddr_storage *addr, int *addrlen, int *sock) {
+static int get_socket(const char *hostname, const char *service, int socktype, struct sockaddr_storage *addr, int *addrlen, int *sock, bool is_output) {
 	struct addrinfo hints, *res, *ressave;
 	int n, ret = -1;
 
@@ -71,11 +71,17 @@ static int bind_addr(const char *hostname, const char *service, int socktype, st
 			int on = 1;
 			setsockopt(*sock, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
 			set_sock_nonblock(*sock);
-			if (bind(*sock, res->ai_addr, res->ai_addrlen) == 0) {
-				memcpy(addr, res->ai_addr, sizeof(*addr));
+			if (is_output) {
+				memcpy(addr, res->ai_addr, res->ai_addrlen);
 				*addrlen = res->ai_addrlen;
 				ret = 0;
-				goto OUT;
+				break;
+			}
+			if (bind(*sock, res->ai_addr, res->ai_addrlen) == 0) {
+				memcpy(addr, res->ai_addr, res->ai_addrlen);
+				*addrlen = res->ai_addrlen;
+				ret = 0;
+				break;
 			} else {
 				char str_addr[INET6_ADDRSTRLEN];
 				my_inet_ntop(res->ai_family, res->ai_addr, str_addr, sizeof(str_addr));
@@ -87,12 +93,19 @@ static int bind_addr(const char *hostname, const char *service, int socktype, st
 		}
 		res = res->ai_next;
 	}
-OUT:
 	freeaddrinfo(ressave);
 
 	return ret;
 }
 
+static int get_input_socket(const char *hostname, const char *service, int socktype, struct sockaddr_storage *addr, int *addrlen, int *sock) {
+	return get_socket(hostname, service, socktype, addr, addrlen, sock, false);
+}
+
+static int get_output_socket(const char *hostname, const char *service, int socktype, struct sockaddr_storage *addr, int *addrlen, int *sock) {
+	return get_socket(hostname, service, socktype, addr, addrlen, sock, true);
+}
+
 static int join_multicast_group(int sock, int ttl, struct sockaddr_storage *addr) {
 	switch (addr->ss_family) {
 	case AF_INET: {
@@ -136,8 +149,12 @@ int udp_connect_input(struct io *io) {
 
 	memset(&addr, 0, sizeof(addr));
 
-	ts_LOGf("Connecting input to %s port %s\n", io->hostname, io->service);
-	if (bind_addr(io->hostname, io->service, SOCK_DGRAM, &addr, &addrlen, &sock) < 0)
+	if (!io->isrc.s_addr)
+		ts_LOGf("Connecting input to %s port %s\n", io->hostname, io->service);
+	else
+		ts_LOGf("Connecting input to %s port %s source %s\n", io->hostname, io->service, inet_ntoa(io->isrc));
+
+	if (get_input_socket(io->hostname, io->service, SOCK_DGRAM, &addr, &addrlen, &sock) < 0)
 		return -1;
 
 	/* Set receive buffer size to ~2.0MB */
@@ -148,6 +165,25 @@ int udp_connect_input(struct io *io) {
 		if (join_multicast_group(sock, io->ttl, &addr) < 0) {
 			close(sock);
 			return -1;
+		} else {
+#ifdef IP_ADD_SOURCE_MEMBERSHIP
+			if (io->isrc.s_addr && addr.ss_family == AF_INET) {
+				/* Source-specific multicast */
+				struct sockaddr_in *src = (struct sockaddr_in *)&addr;
+				struct ip_mreq_source imr;
+				memset(&imr, 0, sizeof(imr));
+				imr.imr_multiaddr = src->sin_addr;
+				imr.imr_sourceaddr = io->isrc;
+				if (setsockopt(sock, IPPROTO_IP, IP_ADD_SOURCE_MEMBERSHIP,
+				    (char *)&imr, sizeof(struct ip_mreq_source)) < 0)
+				{
+					char str_addr[INET6_ADDRSTRLEN];
+					my_inet_ntop(addr.ss_family, (struct sockaddr *)&addr, str_addr, sizeof(str_addr));
+					ts_LOGf("ERROR: Can't set multicast group %s source %s: %s\n",
+						str_addr, inet_ntoa(io->isrc), strerror(errno));
+				}
+			}
+#endif
 		}
 	}
 
@@ -164,9 +200,9 @@ int udp_connect_output(struct io *io) {
 
 	memset(&addr, 0, sizeof(addr));
 
-	ts_LOGf("Connecting output to %s port %s ttl: %d\n",
+	ts_LOGf("Connecting output to %s port %s ttl %d\n",
 		io->hostname, io->service, io->ttl);
-	if (bind_addr(io->hostname, io->service, SOCK_DGRAM, &addr, &addrlen, &sock) < 0)
+	if (get_output_socket(io->hostname, io->service, SOCK_DGRAM, &addr, &addrlen, &sock) < 0)
 		return -1;
 
 	/* Set send buffer size to ~2.0MB */
