@@ -9,16 +9,14 @@
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * GNU General Public License (COPYING file) for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston MA 02110-1301, USA.
  */
 #ifndef UTIL_H
 #define UTIL_H
 
 #include <inttypes.h>
+#include <arpa/inet.h>
 
 unsigned long crc32(unsigned long crc, const uint8_t *buf, unsigned int len);
 int32_t boundary(int32_t exp, int32_t n);
@@ -28,5 +26,8 @@ uint8_t *init_2b(uint32_t val, uint8_t *b);
 void set_thread_name(char *thread_name);
 int decode_hex_string(char *hex, uint8_t *bin, int asc_len);
 int64_t get_time(void);
+unsigned int file_hex2buf(char *filename, uint8_t *buffer, unsigned int buf_size);
+int parse_host_and_port(char *input, char **hostname, char **service, int *port_set);
+char *my_inet_ntop(int family, struct sockaddr *addr, char *dest, int dest_len);
 
 #endif
